<?php
/* 
Description: Core Library Admin Page functions
 
Copyright 2014 Malcolm Shergold

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

require_once "stageshowlib_utils.php";

if (!class_exists('StageShowLibAdminBaseClass')) 
{
	class StageShowLibAdminBaseClass // Define class
  	{
		var $caller;				// File Path of descendent class
		var $myPluginObj;
		var $myDBaseObj;
		var $myDomain;
		
		function __construct($env)	 //constructor	
		{
			$this->caller = $env['caller'];
			$this->myPluginObj = $env['PluginObj'];
			$this->myDBaseObj = $env['DBaseObj'];
			$this->myDomain = $env['Domain'];
		}
		
		function WPNonceField($referer = '', $name = '_wpnonce', $echo = true)
		{
			$this->myDBaseObj->WPNonceField($referer, $name, $echo);
		}
		
		function CheckAdminReferer($referer = '')
		{
			return $this->myDBaseObj->CheckAdminReferer($referer);
		}

		// Bespoke translation functions added to remove these translations from .po file
		function getTL8($text, $domain = 'default') 
		{ 
			return __($text, $domain);
		}
		
		function echoTL8($text, $domain = 'default') 
		{ 
			return _e($text, $domain);
		}
		
  	}
}

if (!class_exists('StageShowLibAdminClass')) 
{
  class StageShowLibAdminClass extends StageShowLibAdminBaseClass// Define class
  {
		var $env;
		
		var $currentPage;		
		var $adminOptions;

		var $editingRecord;
      		
		function __construct($env)	 //constructor	
		{
			parent::__construct($env);
			
			$this->adminOptions = $this->myDBaseObj->adminOptions;
				
			$this->env = $env;
			
			$myPluginObj = $this->myPluginObj;
			$myDBaseObj  = $this->myDBaseObj;
			
			$this->editingRecord = false;			
			$this->pluginName = basename(dirname($this->caller));
			
			if (!isset($this->pageTitle)) $this->pageTitle = "***  pageTitle Undefined ***";
			
			$this->adminMsg = '';			

			$bulkAction = '';
			if ( isset( $_POST['doaction_t'] ) )
			{
				if ( isset( $_POST['action_t'] ) && (-1 != $_POST['action_t']) )
				{
					$bulkAction = $_POST['action_t'];
				}
			}
			
			if ( isset( $_POST['doaction_b'] ) )
			{
				if ( isset( $_POST['action_b'] ) && (-1 != $_POST['action_b']) )
				{
					$bulkAction = $_POST['action_b'];
				}
			}
			
 			if (($bulkAction !== '') && isset($_POST['rowSelect']))
 			{
				// Bulk Action Apply button actions
				$this->CheckAdminReferer();
				
				$actionError = false;
				foreach($_POST['rowSelect'] as $recordId)
				{
					if (!is_numeric($recordId)) break;
					$actionError |= $this->DoBulkPreAction($bulkAction, $recordId);
				}
						
				$actionCount = 0;
				if (!$actionError)
				{
					foreach($_POST['rowSelect'] as $recordId)
					{
						if (!is_numeric($recordId)) break;
						if ($this->DoBulkAction($bulkAction, $recordId))
						{
							$actionCount++;
						}
					}
					// TODO - Entry Updated - Generate iCal file
				}
				
				$actionMsg = $this->GetBulkActionMsg($bulkAction, $actionCount);
				if ($actionCount > 0)
				{
					$this->myDBaseObj->PurgeDB();						
					echo '<div id="message" class="updated"><p>'.$actionMsg.'</p></div>';
				}
				else
				{										
					echo '<div id="message" class="error"><p>'.$actionMsg.'</p></div>';
				}
				
 			}
 			
			$tableClass = $this->myDBaseObj->get_domain().'-admin';			
			echo '<div class="wrap '.$tableClass.'">';

			$this->GetSearchParams();
			
			ob_start();
			$this->ProcessActionButtons();
			$actionPage = ob_get_contents();
			ob_end_clean();
			
			if ($this->pageTitle != '')
			{
				$iconID = 'icon-'.$this->myDomain;
				echo '
					<div id="'.$iconID.'" class="icon32"></div>
					<h2>'.$myDBaseObj->get_pluginName().' - '.__($this->pageTitle, $this->myDomain).'</h2>'."\n";				
			}
			
			echo $actionPage;			
			if (!isset($this->donePage))
			{
				$this->Output_MainPage($this->adminMsg !== '');				
			}
			
			echo '</div>';
		}
		
		static function ValidateEmail($ourEMail)
		{
			if (strpos($ourEMail, '@') === false)
				return false;
				
			return true;
		}

		static function IsOptionChanged($adminOptions, $optionID)
		{
			if (isset($_POST[$optionID]) && (trim(StageShowLibUtilsClass::GetArrayElement($adminOptions, $optionID)) !== trim($_POST[$optionID])))	// TODO: Check for SQLi
			{
				return true;
			}
						
			return false;
		}
		
		function UpdateHiddenRowValues($result, $index, $settings, $dbOpts)
		{
			// Save option extensions
			foreach ($settings as $setting)
			{
				$settingId = $setting[StageShowLibTableClass::TABLEPARAM_ID];
				
				if ($setting[StageShowLibTableClass::TABLEPARAM_TYPE] == StageShowLibTableClass::TABLEENTRY_CHECKBOX)
				{
					$newVal = isset($_POST[$settingId.$index]) ? 1 : 0;
					$_POST[$settingId.$index] = $newVal;
				}
				else if (!isset($_POST[$settingId.$index])) 
					continue;
				else
					$newVal = $_POST[$settingId.$index];	// TODO: Check for SQLi
					
				if ($newVal != $result->$settingId)
				{
					$this->myDBaseObj->UpdateSettings($result, $dbOpts['Table'], $settingId, $dbOpts['Index'], $index);					
				}
			}
		}
		
		function DoBulkPreAction($bulkAction, $recordId)
		{
			return false;
		}
				
		function DoBulkAction($bulkAction, $recordId)
		{
			echo "DoBulkAction() function not defined in ".get_class()."<br>\n";
			return false;
		}
		
		function GetBulkActionMsg($bulkAction, $actionCount)
		{
			echo "GetBulkActionMsg() function not defined in ".get_class()."<br>\n";
		}
		
		function GetAdminListClass()
		{
			$className = get_class($this);
			$derivedClassName = str_replace('AdminClass', 'AdminListClass', $className);
			
			return $derivedClassName;
		}
				
		function OutputPostButton($buttonId, $buttonText, $buttonClass = "button-secondary")
		{
			$clickEvent="return stageshowlib_serialisePost(this, 'stageshowlib_PostVars')";
			$this->OutputButton($buttonId, $buttonText, $buttonClass, $clickEvent);			
		}
		
		function OuputSearchButton($label = '', $buttonId = '')
		{
			if ($label == '') 
			{
				$label = __("Search Sales", $this->myDomain);
				if ($buttonId == '') $buttonId = 'searchsales';
			}
			if ($buttonId == '') $buttonId = strtolower(str_replace(" ", "", $label));
			
			echo '<div class="'.$this->myDomain.'-'.$buttonId.'"><input type="text" maxlength="'.PAYMENT_API_SALEEMAIL_TEXTLEN.'" size="20" name="'.$buttonId.'text" id="'.$buttonId.'text" value="" autocomplete="off" />'."\n";
			$this->OutputButton($buttonId."button", $label);					
			echo '</div>'."\n";
		}
		
		function OutputButton($buttonId, $buttonText, $buttonClass = "button-secondary", $clickEvent = '')
		{
			$buttonText = __($buttonText, $this->myDomain);
			
			if ($clickEvent != '')
			{
				$clickEvent = ' onclick="'.$clickEvent.'" ';
			}
			
			echo "<input class=\"$buttonClass\" type=\"submit\" $clickEvent name=\"$buttonId\" value=\"$buttonText\" />\n";
		}
		
		function Output_MainPage($updateFailed)
		{
			echo "Output_MainPage() function not defined in ".get_class()."<br>\n";
		}
		
		function GetSearchParams()
		{
		}
		
		function ProcessActionButtons()
		{
			echo "ProcessActionButtons() function not defined in ".get_class()."<br>\n";
		}
	
		function AdminUpgradeNotice() 
		{ 
?>
	<div id="message" class="updated fade">
		<p><?php echo '<strong>Plugin is ready</strong>'; ?></p>
	</div>
<?php
		}
	
 	}
}

if (!class_exists('StageShowLibSettingsAdminClass')) 
{
	class StageShowLibSettingsAdminClass extends StageShowLibAdminClass // Define class
	{
		function __construct($env) //constructor	
		{
			$this->pageTitle = 'Settings';
			
			$classId = $this->GetAdminListClass($this);
			$this->adminListObj = new $classId($env, true);			
			
			// Call base constructor
			parent::__construct($env);	
		}
		
		function ProcessActionButtons()
		{
			$myPluginObj = $this->myPluginObj;
			$myDBaseObj = $this->myDBaseObj;			
					
			$SettingsUpdateMsg = '';
				
			if (isset($_POST['savechanges']))
			{
				$this->CheckAdminReferer();
				
				if ($SettingsUpdateMsg === '')
				{
					$this->SaveSettings($myDBaseObj);					
					$myDBaseObj->saveOptions();
					
					echo '<div id="message" class="updated"><p>'.__('Settings have been saved', $this->myDomain).'</p></div>';
				}
				else
				{
					$this->Reload();		
					
					echo '<div id="message" class="error"><p>'.$SettingsUpdateMsg.'</p></div>';
					echo '<div id="message" class="error"><p>'.__('Settings have NOT been saved.', $this->myDomain).'</p></div>';
				}
			}
			
		}
		
		function Output_MainPage($updateFailed)
		{			
			$myPluginObj = $this->myPluginObj;
			$myDBaseObj = $this->myDBaseObj;
			
			// Settings HTML Output - Start 
			
			$formClass = $this->myDomain.'-admin-form';
			echo '<div class="'.$formClass.'">'."\n";
?>	
	<form method="post">
<?php

			$this->WPNonceField();
			
			// Get setting as stdClass object
			$results = $myDBaseObj->GetAllSettingsList();
			if (count($results) == 0)
			{
				echo "<div class='noconfig'>" . __('No Settings Configured', $this->myDomain) . "</div>\n";
			}
			else
			{
				$this->adminListObj->OutputList($results, $updateFailed);
				
				if (!isset($this->adminListObj->editMode) || ($this->adminListObj->editMode))
				{
					if (count($results) > 0)
						$this->OutputButton("savechanges", __("Save Changes", $this->myDomain), "button-primary");				
				}
			}
			
?>
	</form>
	</div>
<?php			
		}
		
		function SaveSettings($dbObj)
		{
			$settingOpts = $this->adminListObj->GetDetailsRowsDefinition();
			
			// Save admin settings to database
			foreach ($settingOpts as $settingOption)
				{	
					if (isset($settingOption[StageShowLibTableClass::TABLEPARAM_READONLY]))
					{
						continue;
					}
					
					switch ($settingOption[StageShowLibTableClass::TABLEPARAM_TYPE])
					{
						case StageShowLibTableClass::TABLEENTRY_READONLY:
						case StageShowLibTableClass::TABLEENTRY_VIEW:
							break;
						
						case StageShowLibTableClass::TABLEENTRY_CHECKBOX:
							$controlId = $settingOption[StageShowLibTableClass::TABLEPARAM_ID];
							$dbObj->adminOptions[$controlId] = isset($_POST[$controlId]) ? true : false;
							break;
						
						case StageShowLibTableClass::TABLEENTRY_TEXT:
						case StageShowLibAdminListClass::TABLEENTRY_DATETIME:
							// Text Settings are "Trimmed"
							$controlId = $settingOption[StageShowLibTableClass::TABLEPARAM_ID];
							if (isset($_POST[$controlId]))
								$dbObj->adminOptions[$controlId] = trim(stripslashes($_POST[$controlId]));	// TODO: Check for SQLi
							break;
						
						default:
							$controlId = $settingOption[StageShowLibTableClass::TABLEPARAM_ID];
							if (isset($_POST[$controlId]))
								$dbObj->adminOptions[$controlId] = stripslashes($_POST[$controlId]);	// TODO: Check for SQLi
							break;
					}
				}	
			
			$dbObj->saveOptions();			
		}
		
		function EditTemplate($templateID, $folder='emails', $isEMail = true)
		{
			$pluginRoot = str_replace('plugins', 'uploads', dirname(dirname(__FILE__)));
			$pluginId = basename($pluginRoot);	

			if (isset($_POST[$templateID.'-Button']))
			{
				$templateFile = $_POST[$templateID];
				$templatePath = "$pluginRoot/$folder/$templateFile";
		
				$editorID = $templateID.'-Editor';
				
				$contents = file_get_contents($templatePath);
				if ($isEMail)
				{
					$pregRslt = preg_match('/(.*[\n])(.*[\n])([\s\S]*?)(\*\/[\s]*?\?\>)/', $contents, $matches);
					if ($pregRslt != 1)
					{
						echo '<div id="message" class="error"><p>' . __('Error parsing file.', $this->myDomain).' - '.$templateFile. '</p></div>';
						$this->donePage = true;
						return true;					
					}
					$subject = $matches[2];
					$contents = $matches[3];
					$htmlMode = (strpos($contents, '</html>') > 0);
					$styles = '';
					if ($htmlMode)	
					{
						// Extract any styles from the source - Editor removes them
						if (preg_match_all('/\<style[\s\S]*?\>([\s\S]*?)\<\/style\>[\s\S]*?/', $contents, $matches) >= 1)
						{
							foreach ($matches[1] as $style)
							{
								$styles .= "\n<style>$style</style>\n";
							}
						}

						$pregRslt = preg_match('/\<body[\s\S]*?\>([\s\S]*?)\<\/body/', $contents, $matches);
						if ($pregRslt != 1)
						{
							echo '<div id="message" class="error"><p>' . __('Error parsing HTML in file', $this->myDomain).' - '.$templateFile. '</p></div>';
							return true;											
						}
						else
						{
							$contents = $matches[1];
						}
						$contents = str_replace("\n", "", $contents);		// Remove all line ends
						$mystyle = '
	<style type="text/css">
	#'.$editorID.'_ifr
	{
		border: solid black 1px;
	}
	</style>';
						$settings = array(
							'wpautop' => false,
						    'editor_css' => $mystyle
						);
					}
					else
					{
						echo '
	<style type="text/css">
	#'.$editorID.'-tmce,
	#qt_'.$editorID.'_toolbar,
	#wp-'.$editorID.'-media-buttons
	{
		display: none;
	}
	</style>';
						$settings = array();
					}								
				}
				else
				{
					// Just need a text editor
					$htmlMode = false;
					$settings = array(
						'wpautop' => true,
					    'media_buttons' => false,
					    'editor_css' => '', 
					    'tinymce' => false,
					    'quicktags' => false
						);
				}
				
				$saveButtonId = $templateID.'-Save';
				$buttonValue = __('Save', $this->myDomain);
				$buttonCancel = __('Cancel', $this->myDomain);
				
				$this->pageTitle .= " ($templateFile)";
				
				echo '<form method="post" id="'.$pluginId.'-fileedit">'."\n";
				if ($isEMail)
				{
					echo "<div id=".$pluginId."-fileedit-div-subject>\n";
					echo __("Subject", $this->myDomain)."&nbsp;<input name=\"$pluginId-fileedit-subject\" id=\"$pluginId-fileedit-subject\" type=\"text\" value=\"$subject\" maxlength=80 size=80 /></div>\n";
				}
				
				wp_editor($contents, $editorID, $settings);
				if ($htmlMode)
				{
					echo "<input name=\"$pluginId-fileedit-html\" id=\"$pluginId-fileedit-html\" type=\"hidden\" value=1 />\n";				
				}	
				echo "<input name=\"$pluginId-fileedit-isEMail\" id=\"$pluginId-fileedit-isEMail\" type=\"hidden\" value=\"$isEMail\" />\n";
				echo "<input name=\"$pluginId-fileedit-name\" id=\"$pluginId-fileedit-name\" type=\"hidden\" value=\"$templateFile\" />\n";
				if (isset($styles)) echo "<input name=\"$pluginId-fileedit-styles\" id=\"$pluginId-fileedit-styles\" type=\"hidden\" value=\"".$styles."\" />\n";
				echo "<input class=\"button-primary\" name=\"$saveButtonId\" id=\"$saveButtonId\" type=\"submit\" value=\"$buttonValue\" />\n";
				echo "<input class=\"button-secondary\" type=\"submit\" value=\"$buttonCancel\" />\n";
				echo "</form>\n";

				$this->donePage = true;
				return true;				
			}				
			
			if (isset($_POST[$templateID.'-Save']))
			{
				if (defined('CORONDECK_RUNASDEMO'))	// Demo Mode: Disable Template Editor (Save)
				{
					echo '<div id="message" class="error"><p>'.__('Cannot Save Templates in Demo Mode', $this->myDBaseObj->get_domain()).'</p></div>';
					return false;
				}
				
				$templateFile = $_POST[$pluginId.'-fileedit-name'];
				$fileParts = pathinfo($templateFile);
				$templateName = $fileParts['filename'];
				$templateExtn = $fileParts['extension'];
				$templateContents = $_POST[$templateID.'-Editor'];
				
				switch($templateExtn)
				{
					case 'css':	
					case 'js':	
						$folderName = $templateExtn;
						$isPHP = false;
						$subject = '';
						break;
						
					default:
						$folderName = 'emails';
						$isPHP = true;
						$subject = stripslashes($_POST[$pluginId.'-fileedit-subject'])."\n";
						if (strpos($subject, '*/') || strpos($templateContents, '*/'))
						{
							echo '<div id="message" class="error"><p>'.__('Template not saved - Invalid Content', $this->myDBaseObj->get_domain()).'</p></div>';
							return false;
						}
					break;					
				}
				$templateFolder = STAGESHOWLIB_UPLOADS_PATH.'/'.$folderName.'/';
				$defaultTemplateFolder = STAGESHOWLIB_DEFAULT_TEMPLATES_PATH.$folderName;
				if (file_exists($defaultTemplateFolder.'/'.$templateFile))
				{
					// The template is a default template - Save with new name
					$fileNumber = 1;
					while (true)
					{
						$destFileName = $templateName."-$fileNumber.".$templateExtn;
						if ($destFileName == $templateFolder.$templateFile)
							break;
						if (!file_exists($templateFolder.$destFileName))
						{
							$templateFile = $destFileName;
							$this->myDBaseObj->adminOptions[$templateID] = $destFileName;
							$this->myDBaseObj->saveOptions();			
							break;							
						}
						$fileNumber++;
						if ($fileNumber > 1000) 
						{
							echo '<div id="message" class="error"><p>' . __('Template Not Saved - Could not rename file.', $this->myDomain).' - '.$templateFile. '</p></div>';
							return false;
						}
					}
				}
				
				$htmlMode = isset($_POST[$pluginId.'-fileedit-html']);
				
				$contents  = '';
				
				if ($isPHP)
				{
					$contents .= "<?php /* Hide template from public access ... Next line is email subject - Following lines are email body\n";
					$contents .= $subject;					
				}
				
				if ($htmlMode)
				{
					$contents .= '
<html>
<head>
<meta http-equiv="content-type" content="text/html; charset=utf-8">';

					if (isset($_POST["$pluginId-fileedit-styles"]))
					{
						$contents .= $_POST["$pluginId-fileedit-styles"];
					}

					$contents .= '
</head>
<body text="#000000" bgcolor="#FFFFFF">';
					$contents .= str_replace("<br />", "<br />\n", stripslashes($templateContents));
					$contents .= '
</body>
</html>
';
				}
				else
				{
					$contents .= stripslashes($_POST[$templateID.'-Editor']);
				}
				
				if ($isPHP)
				{
					$contents .= "\n*/ ?>\n";
				}
				
				file_put_contents($templateFolder.$templateFile, $contents);
				echo '<div id="message" class="updated"><p>' . __('Template Updated.', $this->myDomain).' - '.$templateFile. '</p></div>';
			}				
			
			return false;
		}
		
	}
}

?>